
from hub.persistence import BaseRepo
from hub.persistence.models import City
from hub.persistence.models import HeatPumpSimulation
from hub.persistence.models import User
from hub.persistence.models import UserRoles
from hub.persistence.models import Application
from hub.persistence.models import UserApplications
from hub.persistence.repositories import UserRepo
from hub.hub_logger import logger


class DBSetup:

  def __init__(self, db_name, app_env, dotenv_path):
    """
    Creates database tables and a default admin user
    :param db_name:
    :param app_env:
    :param dotenv_path:
    """
    repo = BaseRepo(db_name=db_name, app_env=app_env, dotenv_path=dotenv_path)
    User.__table__.create(bind=repo.engine, checkfirst=True)
    City.__table__.create(bind=repo.engine, checkfirst=True)
    Application.__table__.create(bind=repo.engine, checkfirst=True)
    UserApplications.__table__.create(bind=repo.engine, checkfirst=True)
    HeatPumpSimulation.__table__.create(bind=repo.engine, checkfirst=True)
    self._user_repo = UserRepo(db_name=db_name, app_env=app_env, dotenv_path=dotenv_path)
    self._create_admin_user(self._user_repo)

  @staticmethod
  def _create_admin_user(user_repo):
    email = 'admin@hub.com'
    password = 'HubAdmin#!98'
    print('Creating default admin user...')
    user = user_repo.insert('Administrator', email, password, UserRoles.Admin)
    if type(user) is dict:
      logger.info(user)
    else:
      print(f'Created Admin user with email: {email}, password: {password} and role: {UserRoles.Admin.value}')
      logger.info(f'Created Admin user with email: {email}, password: {password} and role: {UserRoles.Admin.value}')
      print('Remember to change the admin default password and email address with the UserFactory')



"""
Model representation of the results of heat pump simulation
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Peter Yefi peteryefi@gmail.com
"""

from sqlalchemy import Column, Integer, String, Sequence
from sqlalchemy import Enum, ForeignKey, Float, DateTime
from sqlalchemy.dialects.postgresql import JSONB
from hub.persistence.db_config import Base
import enum
import datetime


class SimulationTypes(enum.Enum):
  Parallel = 'PARALLEL'
  Series = 'SERIES'


class HeatPumpTypes(enum.Enum):
  Air = 'Air Source'
  Water = 'Water to Water'


class HeatPumpSimulation(Base):
  """A model representation of a building

        Attributes:
            city_id,  A reference to the city which was used to run this simulation.
            hourly_electricity_demand, A JSON object that has hours and their electricity demand
            daily_electricity_demand, A JSON object that has days and their electricity demand
            monthly_electricity_demand, A JSON object that has months and their electricity demand
            daily_fossil_fuel_consumption, A JSON object that has days and fossil fuel consumption
            monthly_fossil_fuel_consumption, A JSON object that has months and fossil fuel consumption
            heat_pump_type, Water or air heat pump
            simulation_type, The type of heat pump simulation (parallel or series)
            heat_pump_model, The model of the heat pump (either water to water or air source)
            start year, HP simulation start year
            end year, HP simulation end year
            max_hp_energy_input, Maximum heat pump energy input
            max_demand_storage_hour, Hours of storage at maximum demand
            building_supply_temp, building supply temperature
            temp_difference, Difference in HP and building supply temperatures
            fuel_lhv, The lower heating value of fuel
            fuel_price, The price of fuel
            fuel_efficiency, the efficiency of fuel
            fuel_density, the density of fuel
            hp_supply_temp, supply temperature of heat pump


    """
  __tablename__ = "heat_pump_simulation"
  id = Column(Integer, Sequence('hp_simulation_id_seq'), primary_key=True)
  city_id = Column(Integer, ForeignKey('city.id'), nullable=False)
  daily_electricity_demand = Column(JSONB, nullable=False)
  hourly_electricity_demand = Column(JSONB, nullable=False)
  daily_fossil_fuel_consumption = Column(JSONB, nullable=False)
  monthly_fossil_fuel_consumption = Column(JSONB, nullable=False)
  monthly_electricity_demand = Column(JSONB, nullable=False)
  heat_pump_type = Column(Enum(HeatPumpTypes), nullable=False)
  simulation_type = Column(Enum(SimulationTypes), nullable=False)
  heat_pump_model = Column(String, nullable=False)
  start_year = Column(Integer, nullable=False)
  end_year = Column(Integer, nullable=False)
  max_hp_energy_input = Column(Float, nullable=False)
  max_demand_storage_hour = Column(Float, nullable=False)
  building_supply_temp = Column(Float, nullable=False)
  temp_difference = Column(Float, nullable=False)
  fuel_lhv = Column(Float, nullable=False)
  fuel_price = Column(Float, nullable=False)
  fuel_efficiency = Column(Float, nullable=False)
  fuel_density = Column(Float, nullable=False)
  hp_supply_temp = Column(Float, nullable=False)
  created = Column(DateTime, default=datetime.datetime.utcnow)

  def __init__(self, city_id, hourly_elec_demand, daily_elec_demand, monthly_elec_demand, daily_fossil, monthly_fossil):
    self.city_id = city_id
    self.hourly_electricity_demand = hourly_elec_demand
    self.daily_electricity_demand = daily_elec_demand
    self.monthly_electricity_demand = monthly_elec_demand
    self.daily_fossil_fuel_consumption = daily_fossil
    self.monthly_fossil_fuel_consumption = monthly_fossil



"""
City repository with database CRUD operations
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Peter Yefi peteryefi@gmail.com
"""

from hub.city_model_structure.city import City
from hub.persistence import BaseRepo
from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy import select
from hub.persistence.models import City as DBCity
import pickle
import requests
from urllib3.exceptions import HTTPError
from typing import Union, Dict
from hub.hub_logger import logger
import datetime


class CityRepo(BaseRepo):
  _instance = None

  def __init__(self, db_name: str, dotenv_path: str, app_env: str):
    super().__init__(db_name, dotenv_path, app_env)

  def __new__(cls, db_name, dotenv_path, app_env):
    """
    Implemented for a singleton pattern
    """
    if cls._instance is None:
      cls._instance = super(CityRepo, cls).__new__(cls)
    return cls._instance

  def insert(self, city: City, user_id: int) -> Union[City, Dict]:
    db_city = DBCity(pickle.dumps(city), city.name, city.srs_name, city.country_code, city.lower_corner,
                     city.upper_corner, user_id)
    db_city.climate_reference_city = city.climate_reference_city
    db_city.longitude = city.longitude
    db_city.latitude = city.latitude
    db_city.time_zone = city.time_zone

    try:
      # Retrieve hub project latest release
      response = requests.get("https://rs-loy-gitlab.concordia.ca/api/v4/projects/2/repository/branches/master",
                              headers={"PRIVATE-TOKEN": self.config.hub_token})
      recent_commit = response.json()["commit"]["id"]
      logger.info(f'Current commit of hub is {recent_commit}')
      exiting_city = self._get_by_hub_version(recent_commit, city.name)

      # Do not persist the same city for the same version of Hub
      if exiting_city is None:
        db_city.hub_release = recent_commit
        cities = self.get_by_name(city.name)
        # update version for the same city but different hub versions

        if len(cities) == 0:
          db_city.city_version = 0
        else:
          db_city.city_version = cities[-1].city_version + 1

        # Persist city
        self.session.add(db_city)
        self.session.flush()
        self.session.commit()
        return db_city
      else:
        return {'message': f'Same version of {city.name} exist'}
    except SQLAlchemyError as err:
      logger.error(f'Error while adding city: {err}')
    except HTTPError as err:
      logger.error(f'Error retrieving Hub latest release: {err}')

  def get_by_id(self, city_id: int) -> DBCity:
    """
    Fetch a City based on the id
    :param city_id: the city id
    :return: a city
    """
    try:
      return self.session.execute(select(DBCity).where(DBCity.id == city_id)).first()[0]
    except SQLAlchemyError as err:
      logger.error(f'Error while fetching city: {err}')

  def _get_by_hub_version(self, hub_commit: str, city_name: str) -> City:
    """
    Fetch a City based on the name and hub project recent commit
    :param hub_commit: the latest hub commit
    :param city_name: the name of the city
    :return: a city
    """
    try:
      return self.session.execute(select(DBCity)
                                  .where(DBCity.hub_release == hub_commit, DBCity.name == city_name)).first()
    except SQLAlchemyError as err:
      logger.error(f'Error while fetching city: {err}')

  def update(self, city_id: int, city: City):
    """
    Updates a city
    :param city_id: the id of the city to be updated
    :param city: the city object
    :return:
    """
    try:
      self.session.query(DBCity).filter(DBCity.id == city_id) \
        .update({
          'name': city.name, 'srs_name': city.srs_name, 'country_code': city.country_code, 'longitude': city.longitude,
          'latitude': city.latitude, 'time_zone': city.time_zone, 'lower_corner': city.lower_corner.tolist(),
          'upper_corner': city.upper_corner.tolist(), 'climate_reference_city': city.climate_reference_city,
          'updated': datetime.datetime.utcnow()
        })

      self.session.commit()
    except SQLAlchemyError as err:
      logger.error(f'Error while updating city: {err}')

  def get_by_name(self, city_name: str) -> [DBCity]:
    """
    Fetch city based on the name
    :param city_name: the name of the building
    :return: [ModelCity] with the provided name
    """
    try:
      result_set = self.session.execute(select(DBCity).where(DBCity.name == city_name))
      return [building[0] for building in result_set]
    except SQLAlchemyError as err:
      logger.error(f'Error while fetching city by name: {err}')

  def get_by_user(self, user_id: int) -> [DBCity]:
      """
      Fetch city based on the user who created it
      :param user_id: the id of the user
      :return: [ModelCity] with the provided name
      """
      try:
        result_set = self.session.execute(select(DBCity).where(DBCity.user_id == user_id))
        return [building[0] for building in result_set]
      except SQLAlchemyError as err:
        logger.error(f'Error while fetching city by name: {err}')

  def delete_city(self, city_id: int):
    """
    Deletes a City with the id
    :param city_id: the city id
    :return: a city
    """
    try:
      self.session.query(DBCity).filter(DBCity.id == city_id).delete()
      self.session.commit()
    except SQLAlchemyError as err:
      logger.error(f'Error while fetching city: {err}')



"""
City repository with database CRUD operations
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Peter Yefi peteryefi@gmail.com
"""

from hub.persistence import BaseRepo
from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy import select
from hub.persistence.models import User
from hub.persistence.models import UserRoles
from hub.helpers.auth import Auth
from typing import Union, Dict
from hub.hub_logger import logger
import datetime


class UserRepo(BaseRepo):
  _instance = None

  def __init__(self, db_name: str, dotenv_path: str, app_env: str):
    super().__init__(db_name, dotenv_path, app_env)

  def __new__(cls, db_name, dotenv_path, app_env):
    """
    Implemented for a singleton pattern
    """
    if cls._instance is None:
      cls._instance = super(UserRepo, cls).__new__(cls)
    return cls._instance

  def insert(self, name: str, email: str, password: str, role: UserRoles) -> Union[User, Dict]:
    user = self.get_by_email(email)
    if user is None:
      try:
        if Auth.validate_password(password):
          user = User(name=name, email=email, password=Auth.hash_password(password), role=role)
          self.session.add(user)
          self.session.flush()
          self.session.commit()
          return user
      except SQLAlchemyError as err:
        logger.error(f'An error occurred while creating user: {err}')
    else:
      return {'message': f'user with {email} email already exists'}

  def update(self, user_id: int, name: str, email: str, password: str, role: UserRoles) -> Union[Dict, None]:
    """
    Updates a user
    :param user_id: the id of the user to be updated
    :param name: the name of the user
    :param email: the email of the user
    :param password: the password of the user
    :param role: the role of the user
    :return:
    """
    try:
      if Auth.validate_password(password):
        self.session.query(User).filter(User.id == user_id) \
          .update({'name': name, 'email': email, 'password': Auth.hash_password(password), 'role': role,
                   'updated': datetime.datetime.utcnow()})
      self.session.commit()
    except SQLAlchemyError as err:
      logger.error(f'Error while updating user: {err}')
      return {'err_msg': 'Error occurred while updated user'}

  def get_by_email(self, email: str) -> [User]:
    """
    Fetch user based on the email address
    :param email: the email of the user
    :return: [User] with the provided email
    """
    try:
      return self.session.execute(select(User).where(User.email == email)).first()
    except SQLAlchemyError as err:
      logger.error(f'Error while fetching user by email: {err}')

  def delete_user(self, user_id: int):
    """
    Deletes a user with the id
    :param user_id: the user id
    :return: None
    """
    try:
      self.session.query(User).filter(User.id == user_id).delete()
      self.session.commit()
    except SQLAlchemyError as err:
      logger.error(f'Error while fetching user: {err}')

  def get_user_by_email_and_password(self, email: str, password: str) -> [User]:
    """
    Fetch user based on the email and password
    :param email: the email of the user
    :param password: the password of the user
    :return: [User] with the provided email and password
    """
    try:
      user = self.session.execute(select(User).where(User.email == email)).first()
      if user:
        if Auth.check_password(password, user[0].password):
          return user
        else:
          return {'message': 'Wrong email/password combination'}
      return {'message': 'user not found'}
    except SQLAlchemyError as err:
      logger.error(f'Error while fetching user by email: {err}')



from .city import City
from .heat_pump_simulation import HeatPumpSimulation
from .heat_pump_simulation import SimulationTypes
from .heat_pump_simulation import HeatPumpTypes
from .user import User, UserRoles
from .user_applications import UserApplications
from .application import Application



from .base_repo import BaseRepo
from .repositories.city_repo import CityRepo
from .repositories.heat_pump_simulation_repo import HeatPumpSimulationRepo
from .db_setup import DBSetup
from .repositories.user_repo import UserRepo
from .models.user import UserRoles



"""
Model representation of the user applications
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Guille Gutierrez Guillermo.GutierrezMorote@concordia.ca
"""

import datetime

from sqlalchemy import Column, Integer, ForeignKey, PrimaryKeyConstraint
from sqlalchemy import DateTime

from hub.persistence.db_config import Base


class UserApplications(Base):
  """
  A model representation of the user applications
  """
  __tablename__ = "user_applications"
  user_id = Column(Integer, ForeignKey('user.id'))
  application_id = Column(Integer, ForeignKey('application.id'))

  created = Column(DateTime, default=datetime.datetime.utcnow)
  updated = Column(DateTime, default=datetime.datetime.utcnow)
  __table_args__ = (PrimaryKeyConstraint(user_id, application_id),)

  def __init__(self, user_id, application_id):
    self.user_id = user_id
    self.application_id = application_id



"""
Heat pump simulation repository with database CRUD operations
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Peter Yefi peteryefi@gmail.com
"""

from hub.persistence import BaseRepo, CityRepo
from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy import select
from hub.persistence.models import HeatPumpSimulation
from typing import Union, Dict
from hub.hub_logger import logger


class HeatPumpSimulationRepo(BaseRepo):
  _instance = None

  def __init__(self, db_name, dotenv_path, app_env):
    super().__init__(db_name, dotenv_path, app_env)
    self._city_repo = CityRepo(db_name, dotenv_path, app_env)

  def __new__(cls, db_name, dotenv_path, app_env):
    """
    Implemented for a singleton pattern
    """
    if cls._instance is None:
      cls._instance = super(HeatPumpSimulationRepo, cls).__new__(cls)
    return cls._instance

  def insert(self, hp_sim_data: Dict, city_id: int) -> Union[HeatPumpSimulation, Dict]:
    """
    Inserts the results of heat pump simulation
    :param hp_sim_data: dictionary with heatpump the simulation inputs and output
    :param city_id: the city that was used in running the simulation
    :return: HeatPumpSimulation
    """

    city = self._city_repo.get_by_id(city_id)
    if city is None:
      return {'message': 'city not found in database'}

    try:
      hp_simulation = HeatPumpSimulation(city_id, hp_sim_data["HourlyElectricityDemand"],
                                         hp_sim_data["DailyElectricityDemand"], hp_sim_data["MonthlyElectricityDemand"],
                                         hp_sim_data["DailyFossilFuelConsumption"],
                                         hp_sim_data["MonthlyFossilFuelConsumption"])
      hp_simulation.city_id = city_id
      hp_simulation.end_year = hp_sim_data["EndYear"]
      hp_simulation.start_year = hp_sim_data["StartYear"]
      hp_simulation.max_demand_storage_hour = hp_sim_data["HoursOfStorageAtMaxDemand"]
      hp_simulation.max_hp_energy_input = hp_sim_data["MaximumHPEnergyInput"]
      hp_simulation.building_supply_temp = hp_sim_data["BuildingSuppTemp"]
      hp_simulation.temp_difference = hp_sim_data["TemperatureDifference"]
      hp_simulation.fuel_lhv = hp_sim_data["FuelLHV"]
      hp_simulation.fuel_price = hp_sim_data["FuelPrice"]
      hp_simulation.fuel_efficiency = hp_sim_data["FuelEF"]
      hp_simulation.fuel_density = hp_sim_data["FuelDensity"]
      hp_simulation.hp_supply_temp = hp_sim_data["HPSupTemp"]
      hp_simulation.simulation_type = hp_sim_data["SimulationType"]
      hp_simulation.heat_pump_model = hp_sim_data["HeatPumpModel"]
      hp_simulation.heat_pump_type = hp_sim_data["HeatPumpType"]

      # Persist heat pump simulation data
      self.session.add(hp_simulation)
      self.session.flush()
      self.session.commit()
      return hp_simulation
    except SQLAlchemyError as err:
      logger.error(f'Error while saving heat pump simulation data: {err}')
    except KeyError as err:
      logger.error(f'A required field is missing in your heat pump simulation dictionary: {err}')

  def get_by_id(self, hp_simulation_id: int) -> HeatPumpSimulation:
    """
    Fetches heat pump simulation data
    :param hp_simulation_id: the city id
    :return: a HeatPumpSimulation
    """
    try:
      return self.session.execute(select(HeatPumpSimulation).where(HeatPumpSimulation.id == hp_simulation_id)).first()[
        0]
    except SQLAlchemyError as err:
      logger.error(f'Error while fetching city: {err}')

  def get_by_city(self, city_id: int) -> [HeatPumpSimulation]:
    """
    Fetch heat pump simulation results by city
    :param city_id: the name of the building
    :return: [HeatPumpSimulation] with the provided name
    """
    try:
      result_set = self.session.execute(select(HeatPumpSimulation).where(HeatPumpSimulation.city_id == city_id))
      return [sim_data[0] for sim_data in result_set]
    except SQLAlchemyError as err:
      logger.error(f'Error while fetching city by name: {err}')

  def delete_hp_simulation(self, hp_simulation_id: int):
    """
    Deletes a heat pump simulation results
    :param hp_simulation_id: the heat pump simulation results id
    :return:
    """
    try:
      self.session.query(HeatPumpSimulation).filter(HeatPumpSimulation.id == hp_simulation_id).delete()
      self.session.commit()
    except SQLAlchemyError as err:
      logger.error(f'Error while fetching city: {err}')



"""
Model representation of a City
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Peter Yefi peteryefi@gmail.com
"""

from sqlalchemy import Column, Integer, String, Sequence, ForeignKey
from sqlalchemy import DateTime, PickleType, Float
from hub.persistence.db_config import Base
from sqlalchemy.dialects.postgresql import JSONB
from sqlalchemy.orm import relationship
import datetime
import numpy as np


class City(Base):
  """A model representation of a city
  """
  __tablename__ = "city"
  id = Column(Integer, Sequence('city_id_seq'), primary_key=True)
  city = Column(PickleType, nullable=False)
  name = Column(String, nullable=False)
  srs_name = Column(String, nullable=False)
  climate_reference_city = Column(String, nullable=True)
  time_zone = Column(String, nullable=True)
  country_code = Column(String, nullable=False)
  latitude = Column(Float)
  longitude = Column(Float)
  lower_corner = Column(JSONB, nullable=False)
  upper_corner = Column(JSONB, nullable=False)
  hub_release = Column(String, nullable=False)
  city_version = Column(Integer, nullable=False)
  user_id = Column(Integer, ForeignKey('user.id'))
  user = relationship("User", back_populates="cities")
  created = Column(DateTime, default=datetime.datetime.utcnow)
  updated = Column(DateTime, default=datetime.datetime.utcnow)

  def __init__(self, city, name, srs_name, country_code, l_corner, u_corner, user_id):
    self.city = city
    self.user_id = user_id
    self.name = name
    self.srs_name = srs_name
    self.country_code = country_code
    l_corner = l_corner.tolist() if type(l_corner) == np.ndarray else l_corner
    u_corner = u_corner.tolist() if type(u_corner) == np.ndarray else u_corner
    self.lower_corner = l_corner
    self.upper_corner = u_corner



"""
EnergySystemsFactory retrieve the energy system module for the given region
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Pilar Monsalvete pilar.monsalvete@concordi.
Code contributors: Peter Yefi peteryefi@gmail.com
"""
from pathlib import Path
from hub.imports.energy_systems.air_source_hp_parameters import AirSourceHeatPumpParameters
from hub.imports.energy_systems.water_to_water_hp_parameters import WaterToWaterHPParameters
from hub.helpers.utils import validate_import_export_type
from hub.hub_logger import logger


class EnergySystemsFactory:
  """
  EnergySystemsFactory class
  """

  def __init__(self, handler, city, base_path=None):
    if base_path is None:
      base_path = Path(Path(__file__).parent.parent / 'data/energy_systems')
    self._handler = '_' + handler.lower().replace(' ', '_')
    class_funcs = validate_import_export_type(EnergySystemsFactory)
    if self._handler not in class_funcs:
      err_msg = f"Wrong import type. Valid functions include {class_funcs}"
      logger.error(err_msg)
      raise Exception(err_msg)
    self._city = city
    self._base_path = base_path

  def _air_source_hp(self):
    """
    Enrich the city by using xlsx heat pump information
    """
    AirSourceHeatPumpParameters(self._city, self._base_path).enrich_city()

  def _water_to_water_hp(self):
    """
    Enrich the city by using water to water heat pump information
    """
    WaterToWaterHPParameters(self._city, self._base_path).enrich_city()

  def enrich(self):
    """
    Enrich the city given to the class using the class given handler
    :return: None
    """
    getattr(self, self._handler, lambda: None)()



"""
Model representation of a User
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Peter Yefi peteryefi@gmail.com
"""

from sqlalchemy import Column, Integer, String, Sequence
from sqlalchemy import DateTime, Enum
from hub.persistence.db_config import Base
import datetime
from sqlalchemy.orm import validates
import re
import enum
from sqlalchemy.orm import relationship


class UserRoles(enum.Enum):
  Admin = 'Admin'
  Hub_Reader = 'Hub_Reader'


class User(Base):
  """A model representation of a city
  """
  __tablename__ = "user"
  id = Column(Integer, Sequence('user_id_seq'), primary_key=True)
  name = Column(String, nullable=False)
  email = Column(String, nullable=False, unique=True)
  password = Column(String, nullable=False)
  role = Column(Enum(UserRoles), nullable=False, default=UserRoles.Hub_Reader)
  cities = relationship("City", back_populates="user")
  created = Column(DateTime, default=datetime.datetime.utcnow)
  updated = Column(DateTime, default=datetime.datetime.utcnow)

  @validates("email")
  def validate_email(self, key, address):
    pattern = r'\b[A-Za-z0-9._%+-]+@[A-Za-z0-9.-]+\.[A-Z|a-z]{2,}\b'
    if not re.match(pattern, address):
      raise ValueError("failed simple email validation")
    return address

  def __init__(self, name, email, password, role):
    self.name = name
    self.email = email
    self.password = password
    self.role = role



"""
DBFactory performs database create, delete and update operations
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project CoderPeter Yefi peteryefi@gmail.com
"""
from hub.persistence import CityRepo
from hub.persistence import HeatPumpSimulationRepo
from typing import Dict
from hub.city_model_structure.city import City


class DBFactory:
  """
  DBFactory class
  """

  def __init__(self, db_name, dotenv_path, app_env):
    self._city_repo = CityRepo(db_name=db_name, dotenv_path=dotenv_path, app_env=app_env)
    self._hp_simulation_repo = HeatPumpSimulationRepo(db_name=db_name, dotenv_path=dotenv_path, app_env=app_env)

  def persist_city(self, user_id: int, city: City):
    """
    Persist city into postgres database
    """
    return self._city_repo.insert(city, user_id)

  def update_city(self, city_id, city):
    """
   Update an existing city in postgres database
   :param city_id: the id of the city to update
   :param city: the updated city object
    """
    return self._city_repo.update(city_id, city)

  def delete_city(self, city_id):
    """
    Deletes a single city from postgres
    :param city_id: the id of the city to get
    """
    self._city_repo.delete_city(city_id)

  def persist_hp_simulation(self, hp_simulation_data: Dict, city_id: int):
    """
    Persist heat pump simulation results
    :param hp_simulation_data: the simulation results
    :param city_id: the city object used in running the simulation
    :return: HeatPumpSimulation object
    """
    return self._hp_simulation_repo.insert(hp_simulation_data, city_id)

  def delete_hp_simulation(self, hp_sim_id):
    """
    Deletes a single heat pump simulation from postgres
    :param hp_sim_id: the id of the heat pump simulation to get
    """
    self._hp_simulation_repo.delete_hp_simulation(hp_sim_id)



"""
DBFactory performs read related operations
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project CoderPeter Yefi peteryefi@gmail.com
"""
from hub.persistence import CityRepo
from hub.persistence import HeatPumpSimulationRepo


class DBFactory:
  """
  DBFactory class
  """

  def __init__(self, db_name, app_env, dotenv_path):
    self._city_repo = CityRepo(db_name=db_name, app_env=app_env, dotenv_path=dotenv_path)
    self._hp_simulation_repo = HeatPumpSimulationRepo(db_name=db_name, app_env=app_env, dotenv_path=dotenv_path)

  def get_city(self, city_id):
    """
    Retrieve a single city from postgres
    :param city_id: the id of the city to get
    """
    return self._city_repo.get_by_id(city_id)

  def get_city_by_name(self, city_name):
    """
    Retrieve a single city from postgres
    :param city_name: the name of the city to get
    """
    return self._city_repo.get_by_name(city_name)

  def get_city_by_user(self, user_id):
    """
    Retrieve cities created by user
    :param user_id: the id of the user
    """
    return self._city_repo.get_by_user(user_id)

  def get_hp_simulation(self, hp_sim_id: int):
    """
    Retrieve a single heat pump simulation from postgres
    :param hp_sim_id: the id of the heat pump to get
    """
    return self._hp_simulation_repo.get_by_id(hp_sim_id)

  def get_hp_simulation_by_city(self, city_id: int):
    """
    Retrieve a single city from postgres
    :param city_id: the id of the city
    """
    return self._hp_simulation_repo.get_by_city(city_id)



from setuptools import setup, find_packages
import os.path
import glob

setup(
  name='hub',
  version="0.1",
  packages=find_packages(exclude="unittests"),
  data_files=[
    ('config', [os.path.join('hub/config', 'configuration.ini')]),
    ('greenery', glob.glob('hub/catalog_factories/greenery/ecore_greenery/*.ecore')),
    ('data', glob.glob('hub/data/construction/*.xml')),
    ('data', glob.glob('hub/data/customized_imports/*.xml')),
    ('data', glob.glob('hub/data/energy_systems/*.xml')),
    ('data', glob.glob('hub/data/energy_systems/*.insel')),
    ('data', glob.glob('hub/data/energy_systems/*.xlsx')),
    ('data', glob.glob('hub/data/energy_systems/*.txt')),
    ('data', glob.glob('hub/data/energy_systems/*.yaml')),
    ('data', glob.glob('hub/data/greenery/*.xml')),
    ('data', glob.glob('hub/data/life_cycle_assessment/*.xml')),
    ('data', glob.glob('hub/data/schedules/*.xml')),
    ('data', glob.glob('hub/data/schedules/*.xlsx')),
    ('data', glob.glob('hub/data/schedules/idf_files/*.idf')),
    ('data', glob.glob('hub/data/sensors/*.json')),
    ('data', glob.glob('hub/data/usage/*.xml')),
    ('data', glob.glob('hub/data/usage/*.xlsx')),
    ('data', glob.glob('hub/data/weather/*.dat')),
    ('data', glob.glob('hub/data/weather/epw/*.epw')),
    ('data', glob.glob('hub/data/weather/*.dat'))
  ],
  setup_requires=['setuptools']
)



"""
Model representation of an application
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Guille Gutierrez Guillermo.GutierrezMorote@concordia.ca
"""

import datetime

from sqlalchemy import Column, Integer, String, Sequence
from sqlalchemy import DateTime

from hub.persistence.db_config import Base


class Application(Base):
  """
  A model representation of an application
  """
  __tablename__ = "application"
  id = Column(Integer, Sequence('application_id_seq'), primary_key=True)
  name = Column(String, nullable=False)
  description = Column(String, nullable=False)
  created = Column(DateTime, default=datetime.datetime.utcnow)
  updated = Column(DateTime, default=datetime.datetime.utcnow)

  def __init__(self, name, description):
    self.name = name
    self.description = description



"""
Constant module
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2023 Concordia CERC group
Project Coder Peter Yefi peteryefi@gmail.com
"""

import bcrypt
import re


class Auth(object):

  @staticmethod
  def validate_password(password: str) -> bool:
    """
    Validates a password
    :param password: the password to validate
    :return:
    """
    pattern = "^(?=.*[a-z])(?=.*[A-Z])(?=.*\d)(?=.*[@$!%*#?&])[A-Za-z\d@$!#%*?&]{6,20}$"
    pattern = re.compile(pattern)
    if not re.search(pattern, password):
      raise ValueError("Password must be between 6 to 20 characters and must have at least a number, an uppercase "
                       "letter, a lowercase letter, and a special character")
    return True

  @staticmethod
  def hash_password(password: str) -> str:
    """
    Hashes a password
    :param password: the password to be hashed
    :return:
    """
    return bcrypt.hashpw(password.encode('utf-8'), bcrypt.gensalt(14)).decode('utf-8')

  @staticmethod
  def check_password(password: str, hashed_password) -> bool:
    """
    Hashes a password
    :param password: the password to be checked
    :param hashed_password: the hashed password
    :return:
    """
    return bcrypt.checkpw(password.encode('utf-8'), hashed_password.encode('utf-8'))





